Crate haloumi_lowering

Crate haloumi_lowering 

haloumi-lowering

Core traits for enabling types to integrate with the codegen module.

Modules

error
Error type.
lowerable
Traits for types that can be lowered.

Macros

backend_err
Convenience macro for creating Error::Backend type of errors.
bail_backend
Convenience macro for creating Error::Backend type of errors and immediately returning.
lowering_err
Convenience macro for creating Error::Lowering type of errors.

Traits

ExprLowering
Defines the interface code generators expose for generating expressions in their corresponding IR.
Lowering
Defines the interface code generators expose for generating code in their corresponding IR.

Type Aliases

Result
Result type for lowering related operations.