Module lowerable

Module lowerable 

Traits for types that can be lowered.

Traits

LowerableExpr
Declares that the type can be lowered into an expression.
LowerableStmt
Declares that the type can be lowered into a statement.